/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;
import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**

*/
@SuppressWarnings("serial")
public class InteractiveGRSModels_R6 extends InteractiveFrame
{
   private boolean showMatrix = false;
   private double xTranslation = 0.0;
   private double yTranslation = 0.0;
   private double zRotation = 0.0;
   private double scale = 1.0;

   private Scene scene;
   private List<Model> modelArray = new ArrayList<>();
   private int currentModel = 0;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public InteractiveGRSModels_R6(String title, int fbWidth, int fbHeight)
   {
      super(title, fbWidth, fbHeight);

      // Create the Scene object that we shall render
      scene = new Scene();
      scene.camera.perspective = false;

      // Add two (empty) Positions to the Scene.
      scene.addPosition(new Position(), new Position());

      // Instantiate all the grs models.
      modelArray.add(new GRSModel(new File("assets/grs/bronto_v2.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/rex_v2.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/usa_v2.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/bronto.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/rex.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/usa.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/vinci.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/dragon.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/birdhead.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/knight.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/house.grs")));
      modelArray.add(new GRSModel(new File("assets/grs/scene.grs")));

      // Give each model a random color.
      for (Model m : modelArray)
      {
         ModelShading.setRandomColor(m);
      }

      // Create a set of x and y axes.
      Model axes = new Axes2D(-1, +1, -1, +1, 20, 20);
      ModelShading.setColor(axes, Color.red);

      // Add two models to the Scene.
      scene.positionList.get(0).model = modelArray.get(0);
      scene.positionList.get(1).model = axes;
   }


   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
       //Clip.debug = ! Clip.debug;
       //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
      }
      else if ('a' == c)
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if ('g' == c)
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
         String p = scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('/' == c)
      {
         currentModel = (currentModel + 1) % modelArray.size();
         scene.positionList.get(0).model = modelArray.get(currentModel);
      }
      else if ('?' == c)
      {
         currentModel = (currentModel - 1);
         if (currentModel < 0) currentModel = modelArray.size() - 1;
         scene.positionList.get(0).model = modelArray.get(currentModel);
      }
      else if ('c' == c)
      {
         // Change the solid random color of the current model.
         ModelShading.setRandomColor(scene.positionList.get(0).model);
      }
      else if ('C' == c)
      {
         // Change each color in the current model to a random color.
         ModelShading.setRandomColors(scene.positionList.get(0).model);
      }
      else if ('e' == c && e.isAltDown())
      {
         // Change the random color of each vertex of the current model.
         ModelShading.setRandomVertexColors(scene.positionList.get(0).model);
      }
      else if ('e' == c)
      {
         // Change the solid random color of each edge of the current model.
         ModelShading.setRandomLineSegmentColors(scene.positionList.get(0).model);
      }
      else if ('E' == c)
      {
         // Change the random color of each end of each edge of the current model.
         ModelShading.setRainbowLineSegmentColors(scene.positionList.get(0).model);
      }
      else if ('m' == c)
      {
         showMatrix = ! showMatrix;
      }
      else if ('r' == c)
      {
         scale = 1.0;
         xTranslation = 0.0;
         yTranslation = 0.0;
         zRotation = 0.0;
      }
      else if ('s' == c) // Scale all the models 10% smaller.
      {
         scale /= 1.1;
      }
      else if ('S' == c) // Scale all the models 10% larger.
      {
         scale *= 1.1;
      }
      else if ('x' == c)
      {
         xTranslation -= 0.01;
      }
      else if ('X' == c)
      {
         xTranslation += 0.01;
      }
      else if ('y' == c)
      {
         yTranslation -= 0.01;
      }
      else if ('Y' == c)
      {
         yTranslation += 0.01;
      }
      else if ('w' == c)
      {
         zRotation -= 1.0;
      }
      else if ('W' == c)
      {
         zRotation += 1.0;
      }

      // Set the model-to-camera transformation matrix.
      // The order of the transformations is very important!
      Position model_p = scene.positionList.get(0);
      // Move the model relative to its new position.
      model_p.matrix = Matrix.translate(xTranslation,
                                        yTranslation,
                                        0).mult(
                       Matrix.rotateZ(zRotation)).mult(
                       Matrix.scale(scale));

      if (showMatrix && ('m'==c||'r'==c
           ||'s'==c||'x'==c||'y'==c||'w'==c
           ||'S'==c||'X'==c||'Y'==c||'W'==c))
      {
         System.out.println("zRot = " + zRotation);
         System.out.print( model_p.matrix );
      }

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbp.update();
      repaint();
   }


   // Implement part of the ComponentListener interface.
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = this.fbp.getWidth();
      int h = this.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h);
      this.fbp.setFrameBuffer(fb);
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbp.update();
      repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
      int width  = 1024;
      int height = 1024;
      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions. NOTE: We need to call the
      // InteractiveModelsAll_R6 constructor in the Java GUI Event
      // Dispatch Thread, otherwise we get a race condition
      // between the constructor (running in the main() thread)
      // and the very first ComponentEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // call the constructor that builds the gui
               new InteractiveGRSModels_R6("Renderer 6", width, height);
            }
         }
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the '/' key to cycle through the models.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, keys to translate the models along the x and y axes.");
      System.out.println("Use the w/W, key to rotate the models around the z axis.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'c' key to change the random solid model color.");
      System.out.println("Use the 'C' key to randomly change model's colors.");
    //System.out.println("Use the 'e' key to change the random vertex colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'g' key to toggle gamma correction on and off.");
      System.out.println("Use the 'm' key to toggle showing the Model transformation matrix.");
      System.out.println("Use the 'r' key to reset the transformation matrix to the identity.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
