/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   Use this program to experiment with two-dimensional transformations.
<p>
   Try mixing the ordering of rotations, translations, and scales.
*/
public class TransformationExample
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Use orthographic projection
      scene.camera.perspective = false;

      // Create a 2D Model object.
      Model m = new GRSModel(new File("assets/grs/bronto_v2.grs"));

      // Set the model's color.
      ModelShading.setColor(m, Color.black);

      // Add the model to the Scene.
      Position position = new Position(m);
      scene.positionList.add(position);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-1, +1, -1, +1, 24, 24);
      ModelShading.setColor(axes, Color.red);
      scene.positionList.add(new Position(axes));

      // Draw a circle.
      Model circle = new Circle(0.5, 60);
      ModelShading.setColor(circle, Color.blue);
      scene.positionList.add(new Position(circle));

      // Create a framebuffer to render our scene into.
      int vp_width  = 800;
      int vp_height = 800;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      for (int i = 0; i <= 36; i++)
      {
         position.matrix2Identity();
         // try changing the order of these transformations
         position.matrix.mult( Matrix.scale(0.5) );// scale the model to fit in the view rectangle
         position.matrix.mult( Matrix.rotateZ(10*i) );
         position.matrix.mult( Matrix.translate(1, 0, 0) );

         // Render again.
         fb.clearFB(Color.lightGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_TransformationExample_a_Frame%02d.ppm", i));
      }

      for (int i = 0; i <= 36; i++)
      {
         position.matrix2Identity();
         // try changing the order of these transformations
         position.matrix.mult( Matrix.scale(0.5) );// scale the model to fit in the view rectangle
         position.matrix.mult( Matrix.translate(1, 0, 0) );
         position.matrix.mult( Matrix.rotateZ(10*i) );

         // Render again.
         fb.clearFB(Color.lightGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_TransformationExample_b_Frame%02d.ppm", i));
      }

      for (int i = 0; i <= 36; i++)
      {
         position.matrix2Identity();
         // try changing the order of these transformations
         position.matrix.mult( Matrix.scale(0.5) );// scale the model to fit in the view rectangle
         position.matrix.mult( Matrix.translate(Math.cos(i*Math.PI/18), 0, 0) );
         position.matrix.mult( Matrix.translate(0, Math.sin(i*Math.PI/18), 0) );

         // Render again.
         fb.clearFB(Color.lightGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_TransformationExample_c_Frame%02d.ppm", i));
      }
   }
}