/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its upper right hand corner.
<p>
   Do this by "accumulating" rotations.
*/
public class Example_05b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Create a Position for the axes.
      Position axes_p = new Position(axes);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.matrix = Matrix.translate(0, 0, -2);

      // Create a Model of a square.
      Model square = new Model();
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.magenta,
                      Color.green);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Create a Position for the square.
      Position square_p = new Position(square);
      // Add the square to the Scene.
      scene.addPosition(square_p);

      // Push the square away from where the camera is.
      square_p.matrix = Matrix.translate(0, 0, -2);

      //  Rotate the square around its upper right hand corner.
      //  Do this by "accumulating" rotations.
      square_p.matrix.mult( Matrix.translate(1.0, 1.0, 0) );
      Matrix save = square_p.matrix;
      for (int i = 0; i < 60; i++)
      {
         square_p.matrix.mult( Matrix.translate(-1.0, -1.0, 0) );

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_05b_Frame%03d.ppm", i));

         square_p.matrix = save;
         square_p.matrix.mult( Matrix.rotate(6, 0,0,1) ); // accumulate a rotation
         save = square_p.matrix;
      }
   }
}
