/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around the point (3/4, 3/4) in the
   square's model coordinate system, but with the center of
   rotation at the point (1/4, 1/4) in the camera's view
   coordinate system.
<p>
   Also demonstrate the transformation of the coordinate system.
*/
public class Example_07b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Create a Position for the axes.
      Position axes_p = new Position(axes);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.matrix = Matrix.translate(0, 0, -2);

      // Create three shorter sets of x and y axes.
      Model sAxes1 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes3 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      ModelShading.setColor(sAxes1, new Color(150, 0, 0));
      ModelShading.setColor(sAxes2, new Color(100, 0, 0));
      ModelShading.setColor(sAxes3, new Color( 50, 0, 0));
      Position sAxes1_p = new Position(sAxes1);
      Position sAxes2_p = new Position(sAxes2);
      Position sAxes3_p = new Position(sAxes3);
      // Add the shorter axes to the Scene.
      scene.addPosition(sAxes1_p, sAxes2_p, sAxes3_p);

      // Create a Model of a square.
      Model square = new Model();
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.magenta,
                      Color.green);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Create a Position for the square.
      Position square_p = new Position(square);
      // Add the square to the Scene.
      scene.addPosition(square_p);

      // Rotate the square around the point (3/4, 3/4) in the
      // square's model coordinate system, but with the center of
      // rotation at the point (1/4, 1/4) in the camera's view
      // coordinate system.
      for (int i = 0; i < 60; i++)
      {
         sAxes1_p.matrix = Matrix.translate(0, 0, -2);
         sAxes2_p.matrix = Matrix.translate(0, 0, -2);
         sAxes3_p.matrix = Matrix.translate(0, 0, -2);

         // Push the square away from where the camera is.
         square_p.matrix = Matrix.translate(0, 0, -2);

         sAxes1_p.matrix.mult( Matrix.translate(0.25, 0.25, 0) );

         sAxes2_p.matrix.mult( Matrix.translate(0.25, 0.25, 0) );
         sAxes2_p.matrix.mult( Matrix.rotate(6*i, 0,0,1) );

         sAxes3_p.matrix.mult( Matrix.translate(0.25, 0.25, 0) );
         sAxes3_p.matrix.mult( Matrix.rotate(6*i, 0,0,1) );
         sAxes3_p.matrix.mult( Matrix.translate(-0.75, -0.75, 0) );

         square_p.matrix.mult( Matrix.translate(0.25, 0.25, 0) );
         square_p.matrix.mult( Matrix.rotate(6*i, 0,0,1) );
         square_p.matrix.mult( Matrix.translate(-0.75, -0.75, 0) );

         sAxes1.visible = false;
         sAxes2.visible = false;
         sAxes3.visible = false;
         square.visible = false;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_07b_Frame%03da.ppm", i));

         sAxes1.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_07b_Frame%03db.ppm", i));

         sAxes2.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_07b_Frame%03dc.ppm", i));

         sAxes3.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_07b_Frame%03dd.ppm", i));

         square.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_07b_Frame%03de.ppm", i));
      }
   }
}
