/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Translate the letter {@link P} along the y-axis while
   rotating around the letter's vertical center line (y=0.5).
<p>
   See the animation <a href="../Example_25.gif">Example_25.gif</a>.
<p>
   <a href="../Example_25.gif"><img src="../Example_25.gif" alt="Example_25.gif"></a>
*/
public class Example_25
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-5, +5, -5, +5, 20, 20);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.matrix = Matrix.translate(0, 0, -5);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Translate along the y-axis while rotating around
      // the letter's vertical center line (y=0.5).
      for (int i = 0; i <= 50; i++)
      {
         // Push the model P away from where the camera is.
         modelP_p.matrix = Matrix.translate(0, 0, -5);
         // Rotate P around its center axis.
         modelP_p.matrix.mult( Matrix.translate(0.5, 0, 0) );
         modelP_p.matrix.mult( Matrix.rotate(i*36, 0,1,0) );
         modelP_p.matrix.mult( Matrix.translate(-0.5, 0, 0) );
         // Translate P along the y-axis.
         modelP_p.matrix.mult( Matrix.translate(0, i*0.1, 0) );

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_25_Frame%03d.ppm", i));
      }
   }
}
