/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} while scaling it twice as large for each rotation.
<p>
   See the animation <a href="../Example_31.gif">Example_31.gif</a>.
<p>
   <a href="../Example_31.gif"><img src="../Example_31.gif" alt="Example_31.gif"></a>
*/
public class Example_31
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-4, +4, -4, +4, 16, 16);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.matrix = Matrix.translate(0, 0, -4);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P while scaling it
      // twice as large for each rotation.
      for (int i = 0; i <= 120; i++)
      {
         // Push the model away from where the camera is.
         modelP_p.matrix = Matrix.translate(0, 0, -4);

         modelP_p.matrix.mult( Matrix.rotate(i*6, 0,0,1) );
         double scale = Math.pow(2, i/60.0);
         modelP_p.matrix.mult( Matrix.scale(scale, scale, 1) );

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_31_Frame%03d.ppm", i));
      }
   }
}
