/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} down the x-axis as it grows larger.
<p>
   See the animation <a href="../Example_36_puzzle.gif">Example_36_puzzle.gif</a>.
<p>
   <a href="../Example_36_puzzle.gif"><img src="../Example_36_puzzle.gif" alt="Example_36.gif"></a>
*/
public class Example_36
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-11, +11, -11, +11, 22, 22);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.matrix = Matrix.translate(0, 0, -11);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P down the x-axis as it grows larger.

      // do the first 90 degrees in steps of 3 degrees
      for (int i = 0; i <= 30; i++)
      {
         // Undo all the previous transformations and
         // push the model P away from where the camera is.
         modelP_p.matrix = Matrix.translate(0, 0, -11);




         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", i));
      }


      // do the next 180 degrees in steps of 3 degrees
      for (int i = 1; i <= 60; i++)
      {
         // Undo all the previous transformations and
         // push the model P away from where the camera is.
         modelP_p.matrix = Matrix.translate(0, 0, -11);





         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", 30+i));
      }


      // do the last 90 degrees in steps of 3 degrees
      for (int i = 1; i <= 30; i++)
      {
         // Undo all the previous transformations and
         // push the model P away from where the camera is.
         modelP_p.matrix = Matrix.translate(0, 0, -11);





         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_36_Frame%03d.ppm", 90+i));
      }
   }
}
