/*
   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html#mode=edit
*/

public class VisualizeMatrixOperations
{
   public static void main(String[] args) {
      Matrix2 m1 = new Matrix2(2, 0, 0, 2);
      Matrix2 m2 = new Matrix2(3, 0, 0, 3);
      Matrix2 m3 = new Matrix2(4, 0, 0, 4);
      // The following two line of code act very differently!
      Matrix2 m4 = m3.times(m2).times(m1);
      Matrix2 m5 = m3.mult(m2).mult(m1);  // Bad idea! (Why?)
      Matrix2 m6 = m3.mult(m2.mult(m1));  // Even worse! (Why?)
      // Why is the next line a good idea?
      Matrix2 m7 = new Matrix2(1,0,0,1).mult(m3).mult(m2).mult(m1);
   }
}


class Matrix2
{
   private double m00, m01, m10, m11;

   public Matrix2(double m00, double m01, double m10, double m11) {
      this.m00 = m00;
      this.m01 = m01;
      this.m10 = m10;
      this.m11 = m11;
   }

   public Matrix2 times(Matrix2 m) {
      double a00 = this.m00 * m.m00 + this.m01 * m.m10;
      double a01 = this.m00 * m.m01 + this.m01 * m.m11;
      double a10 = this.m10 * m.m00 + this.m11 * m.m10;
      double a11 = this.m10 * m.m10 + this.m11 * m.m11;
      return new Matrix2(a00, a01, a10, a11);
   }

   public Matrix2 mult(Matrix2 m) {
      double a00 = this.m00 * m.m00 + this.m01 * m.m10;
      double a01 = this.m00 * m.m01 + this.m01 * m.m11;
      double a10 = this.m10 * m.m00 + this.m11 * m.m10;
      double a11 = this.m10 * m.m10 + this.m11 * m.m11;
      this.m00 = a00;
      this.m01 = a01;
      this.m10 = a10;
      this.m11 = a11;
      return this;
   }
}

/*
Notice the following important failing of the Java language.

The times() method has this signature,

   public Matrix2 times(Matrix2 m)

and it is applied like this,

   m3.times(m2).times(m1)

The mult() method has essentially the same signature,

   public Matrix2 mult(Matrix2 m)

and it is applied in essentially the same way,

   m3.mult(m2).mult(m1)

So there is nothing in the Java syntax, nor in the Java
type system, that helps you realize that mult() mutates
the object it is called on, while times() returns a new
object. You can only tell the difference between the
two methods be reading the details of their source code.

Another way to put this is that a method's interface,
its signature,

   public Matrix2 times(Matrix2 m)
   public Matrix2  mult(Matrix2 m)

tells you very little about what the method does. The
interface of a method does not work as a specification.
*/
